"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const boa = require('@pipcook/boa');
const { tuple } = boa.builtins();
const tf = boa.import('tensorflow');
const AUTOTUNE = tf.data.experimental.AUTOTUNE;
function loadImage(path) {
    let image = tf.io.read_file(path);
    image = tf.image.decode_jpeg(image, boa.kwargs({
        channels: 3
    }));
    return image;
}
function createDataset(dataLoader, labelMap) {
    return __awaiter(this, void 0, void 0, function* () {
        const imageNames = [];
        const labels = [];
        const count = yield dataLoader.len();
        for (let i = 0; i < count; i++) {
            const currentData = yield dataLoader.getItem(i);
            imageNames.push(currentData.data);
            labels.push(tf.one_hot(currentData.label.categoryId, Object.keys(labelMap).length));
        }
        console.log(imageNames);
        const pathDs = tf.data.Dataset.from_tensor_slices(imageNames);
        const imageDs = pathDs.map(loadImage, boa.kwargs({
            num_parallel_calls: AUTOTUNE
        }));
        const labelDs = tf.data.Dataset.from_tensor_slices(labels);
        const imageLabelDs = tf.data.Dataset.zip(tuple([imageDs, labelDs]));
        return imageLabelDs;
    });
}
/**
 * this is plugin used to train tfjs model with pascal voc data format for image classification problem.
 * @param data : train data
 * @param model : model loaded before
 * @param epochs : need to specify epochs
 * @param batchSize : need to specify batch size
 * @param optimizer : need to specify optimizer
 */
const ModelTrain = (data, model, args) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { epochs = 10, batchSize = 16, saveModel } = args;
        const { trainLoader, validationLoader, metadata } = data;
        let trainDataSet = yield createDataset(trainLoader, metadata.labelMap);
        const count = yield trainLoader.len();
        trainDataSet = trainDataSet.repeat();
        trainDataSet = trainDataSet.batch(batchSize);
        trainDataSet = trainDataSet.prefetch(boa.kwargs({
            buffer_size: AUTOTUNE
        }));
        const trainConfig = {
            epochs: epochs,
            steps_per_epoch: parseInt(String(count / batchSize))
        };
        if (validationLoader) {
            console.log('create validation dataset');
            let validationDataSet = yield createDataset(validationLoader, metadata.labelMap);
            const valCount = yield validationLoader.len();
            validationDataSet = validationDataSet.batch(batchSize);
            trainConfig.validation_data = validationDataSet;
            trainConfig.validation_steps = parseInt(String(valCount / batchSize));
        }
        const trainModel = model.model;
        console.log('start ...');
        yield trainModel.fit(trainDataSet, boa.kwargs(trainConfig));
        yield saveModel((modelPath) => __awaiter(void 0, void 0, void 0, function* () {
            yield trainModel.save_weights(modelPath);
        }));
        const result = Object.assign(Object.assign({}, model), { model: trainModel });
        return result;
    }
    catch (err) {
        console.error('occurs an error on model trainer', err);
        throw err;
    }
});
exports.default = ModelTrain;
//# sourceMappingURL=index.js.map